# -*- coding: utf8 -*-
from __future__ import unicode_literals

import os
import smbc
import time
import fnmatch

from driverng.transport.base import BaseFileBackend, BaseFileTransport


class Samba(BaseFileBackend):
    def __init__(
        self,
        hostname,
        domain="WORKGROUP",
        username=None,
        password=None,
        logger=None,
        name_ok=None,
        glob="*.*",
        ignore_date=False,
    ):
        self.hostname = hostname
        self.username = username
        self.password = password
        self.domain = domain
        self.logger = logger
        self.name_ok = name_ok
        self.glob = glob
        self._ttl = time.time()
        self.ignore_date = ignore_date

        def auth(*args):
            return (self.domain, self.username, self.password)

        self.ctx = smbc.Context(auth_fn=auth)

        self.logger.info(
            "Connected to smb://%s@%s using domain %s",
            self.username,
            self.hostname,
            self.domain,
        )

        super(Samba, self).__init__()

    def _get_smb_url(self, path):
        if path[0] == "/":
            path = path[1:]
        return "smb://{}/{}".format(self.hostname, path)

    def open(self, path, *args, **kwargs):
        return self.ctx.open(self._get_smb_url(path), *args, **kwargs)

    def list(self, path):
        files = []
        for entry in self.ctx.opendir(self._get_smb_url(path)).getdents():
            if entry.smbc_type == smbc.FILE:
                invalide = False
                if fnmatch.fnmatch(entry.name, self.glob):
                    file_path = os.path.join(path, entry.name)
                    try:
                        distant_file = self.open(file_path)
                    except smbc.NoEntryError:
                        self.logger.warning("Fichier inexistant (%s)", file_path)
                        continue
                    st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime = (
                        distant_file.fstat()
                    )
                    if (
                        self.name_ok is None
                        or entry.name.split(".")[-1] != self.name_ok
                    ):
                        if st_size == 0:
                            self.logger.warning("Fichier (%s) vide", entry.name)
                            invalide = True
                        elif (time.time() - st_mtime < 5) and not self.ignore_date:
                            self.logger.warning(
                                "Fichier (%s) modification il y a moins de 5s",
                                entry.name,
                            )
                            invalide = True
                            if time.time() - st_mtime < -30:
                                self.logger.info(
                                    "Fichier (%s) modification au moins 30s dans le futur, veuillez vérifier l'heure système du PC de l'analyseur sur lequel est le répertoire partagé",
                                    entry.name,
                                )

                    if not invalide:
                        files.append(entry.name)
                    distant_file.close()
        return files

    def put(self, data, path):
        self.logger.info("samba put %s", path)
        try:
            self.open(path)
            raise RuntimeError("file exist")
        except smbc.NoEntryError:
            pass

        remote_file = self.open(path, os.O_CREAT | os.O_WRONLY)
        remote_file.write(data)
        remote_file.close()
        #  tester si le fichier existe, implémenter le rename

    def get(self, path):
        try:
            remote_file = self.open(path)
        except smbc.NoEntryError:
            self.logger.warning("Fichier inexistant (%s)", path)
            return None
        data = remote_file.read()
        remote_file.close()
        return data

    def delete(self, path):
        self.ctx.unlink(self._get_smb_url(path))


class SambaTransport(BaseFileTransport):
    def __init__(
        self, domain="WORKGROUP", hostname=None, username=None, password=None, **kwargs
    ):
        backend = Samba(
            hostname,
            domain,
            username,
            password,
            logger=kwargs.get("logger"),
            name_ok=kwargs.get("name_ok", None),
            glob=kwargs.get("glob", "*.*"),
            ignore_date=kwargs.get("ignore_date", False),
        )
        super(SambaTransport, self).__init__(backend, **kwargs)
