# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

import pytest

import driverng
import driverng.exceptions as exceptions
from driverng.tests import unindent

parser_class = "easylink"


def test_decode_order(self):
    worklist = self.decode(
        """
    H|\\^&|||SIL|||||EL||P|DB 100102|20151103152956
    P|1||02028357||NOM^Jessie^^||19931220|M||||06 45 65 15 21|||||||||||20151103||^|||||||
    O|1|2072841||^^^1607^2202^K\\^^^1903^2202^Na\\^^^2101^2202^Urée\\^^^4027^2202^Créatinine\\^^^0127^2202^ASAT\\^^^0146^2202^ALAT\\^^^0312^2202^Cl\\^^^0320^2202^CRP|R||20151103153000||||A||||P|||1|||||||O
    L|1|N"""
    )
    expected = {
        "context": "order",
        "sender_id": "SIL",
        "receiver_id": "EL",
        "samples": [
            {
                "type": "patient",
                "sample_id": "2072841",
                "request_id": "2072841",
                "admission_date": datetime.datetime(2015, 11, 3, 0, 0),
                "birthdate": datetime.datetime(1993, 12, 20, 0, 0),
                "collection_date": datetime.datetime(2015, 11, 3, 15, 30),
                "first_name": "Jessie",
                "height": "",
                "last_name": "NOM",
                "location": "",
                "location_name": "",
                "orders": [
                    {"code": "1607P", "action": "add", "priority": "R"},
                    {"code": "1903P", "action": "add", "priority": "R"},
                    {"code": "2101P", "action": "add", "priority": "R"},
                    {"code": "4027P", "action": "add", "priority": "R"},
                    {"code": "0127P", "action": "add", "priority": "R"},
                    {"code": "0146P", "action": "add", "priority": "R"},
                    {"code": "0312P", "action": "add", "priority": "R"},
                    {"code": "0320P", "action": "add", "priority": "R"},
                ],
                "patient_id": "02028357",
                "sex": "M",
                "specimen": "P",
                "weight": "",
            }
        ],
    }
    assert worklist == expected


def test_decode_order_no_ipp(self):
    worklist = self.decode(
        """
    H|\\^&|||SIL|||||EL||P|DB 100102|20151103152956
    P|1||||NOM^Jessie^^||19931220|M||||06 45 65 15 21|||||||||||20151103||^|||||||
    O|1|2072841||^^^1607|R||20151103153000||||A||||P|||1|||||||O
    L|1|N"""
    )
    expected = {
        "context": "order",
        "sender_id": "SIL",
        "receiver_id": "EL",
        "samples": [
            {
                "type": "patient",
                "sample_id": "2072841",
                "request_id": "2072841",
                "admission_date": datetime.datetime(2015, 11, 3, 0, 0),
                "birthdate": datetime.datetime(1993, 12, 20, 0, 0),
                "collection_date": datetime.datetime(2015, 11, 3, 15, 30),
                "first_name": "Jessie",
                "height": "",
                "last_name": "NOM",
                "location": "",
                "location_name": "",
                "orders": [{"code": "1607P", "action": "add", "priority": "R"}],
                "patient_id": "IPP2072841",
                "sex": "M",
                "specimen": "P",
                "weight": "",
            }
        ],
    }
    assert worklist == expected


def test_encode_result(self):
    result_worklist = copy.deepcopy(self.result_worklist)
    codes = ["0127P", "0312P", "0320P"]
    result_worklist.samples[0].rack = "RACK-POS"
    for i, result in enumerate(result_worklist.samples[0].results):
        result.code = codes[i]
        result.operator = "VAL"
    message = self.result(worklist=result_worklist)
    expected = """
    H|\\^&|||EL|||||SIL||P|DB 100102|20130715141912|
    P|1||L036666666||NYME^ANO||20110513121500|F||||||||1.70|58||||||||LOC^LOCATION|
    O|1|123456||^^^0127^2202\\^^^0312^2202\\^^^0320^2202|R||20130513114242|20130513124242|||N||||P||||3||||||F|
    C|1|I|G001^RACK-POS|G|
    R|1|^^^0127^2202|42|mV||||F||VAL||20110506121400|INST|
    R|2|^^^0312^2202|NORESULT|mV||||F||VAL||20110506121400|INST|
    C|1|I|LL|E|
    C|2|I|<|E|
    R|3|^^^0320^2202|42|mV||||F||VAL||20110506121400|INST|
    L|1|N|
    """
    assert message == expected


def test_encode_result_erreur(self):
    result_worklist = copy.deepcopy(self.result_worklist)
    codes = ["0127P", "0312P", "0320S"]
    for i, result in enumerate(result_worklist.samples[0].results):
        result.code = codes[i]
    with pytest.raises(exceptions.ParametrageError):
        self.result(worklist=result_worklist)


def test_encode_control(self):
    result_control_worklist = copy.deepcopy(self.result_control_worklist)
    result_control_worklist.samples[0].specimen = "U"
    message = self.result_control(worklist=result_control_worklist)
    expected = """
    H|\\^&|||EL|||||SIL||Q|DB 100102|20130715141912|
    P|1|||||||U|
    O|1|CQ123456||^^^A^2202\\^^^B^2202|R||||||N||||U^QC1||||2||||||F|
    R|1|^^^A^2202|42|mV||||F||auto||20110506121400|INST|
    C|1|I|HH|E|
    C|2|I|>|E|
    R|2|^^^B^2202|4.40|||||F||auto||20110506121300|INST|
    C|1|I|R001^REAGENT|R|
    C|2|I|R002^201206061200|R|
    C|3|I|Q001^CONTROL|Q|
    C|4|I|C002^CALIBRATION|C|
    O|2|CQ123488||^^^A^2202|R||||||N||||^QC1||||1||||||F|
    R|1|^^^A^2202|4|mV||||F||auto||20110506121400|INST|
    L|1|N|
    """
    assert message == expected


def test_encode_control_vista(self):
    vista = driverng.get("vista")()
    worklist = vista.decode(
        unindent(
            """
    H|\\^&|1||DV330574|||||HOST||Q|DB 100102|20130311133252|
    P|1||||^^^|||U|||||^^^^|||||^|||||\\||^||||||||||
    O|1|45633VB||^^^0106^2202^Albumin\\^^^0127^2202^Aspartate Aminotransferase\\^^^2101^2202^Blood Urea Nitrogen|R||||||Q||||S^QC3|^^^^||1|26|||20130311133252|||F|||||
    R|1|^^^0106^2202^Albumin|41.56|g/L|38.9 - 44.8|||F||ML|20130311131315|20130311131615|VistaDim1
    C|1|I|R001^12244AB|R
    C|2|I|R002^20130831235900|R
    C|3|I|R003^000666|R
    C|4|I|Q001^45633VB|Q
    C|5|I|Q002^20140228000000|Q
    C|6|I|Q004^ASYD MULTIQUAL|Q
    C|7|I|Q005^A|Q
    R|2|^^^0127^2202^Aspartate Aminotransferase|252.19|U/L|222.0 - 267.0|||F||ML|20130311131116|20130311131745|VistaDim1
    C|1|I|R001^12208BG|R
    C|2|I|R002^20130726235900|R
    C|3|I|R003^004271|R
    C|4|I|Q001^45633VB|Q
    C|5|I|Q002^20140228000000|Q
    C|6|I|Q004^ASYD MULTIQUAL|Q
    C|7|I|Q005^A|Q
    R|3|^^^2101^2202^Blood Urea Nitrogen|1.37|g/L|1.34 - 1.61|||F||ML|20130311131355|20130311131658|VistaDim1
    C|1|I|E550^Correlated|E
    C|2|I|R001^12292BA|R
    C|3|I|R002^20131018235900|R
    C|4|I|R003^001135|R
    C|5|I|Q001^45633VB|Q
    C|6|I|Q002^20140228000000|Q
    C|7|I|Q004^ASYD MULTIQUAL|Q
    C|8|I|Q005^A|Q
    L|1|N"""
        )
    )
    message = self.encode(worklist)
    expected = """
    H|\\^&|||EL|||||SIL||Q|DB 100102|20130715141912|
    P|1|||||||U|
    O|1|45633VB||^^^0106^2202\\^^^0127^2202\\^^^2101^2202|R||||||N||||S^QC3||||3||||||F|
    R|1|^^^0106^2202|41.56|g/L|38.9 - 44.8|||F||ML||20130311131615|VistaDim1|
    C|1|I|Q001^45633VB|Q|
    C|2|I|Q004^ASYD MULTIQUAL|Q|
    C|3|I|Q002^201402280000|Q|
    C|4|I|R001^12244AB|R|
    C|5|I|R002^201308312359|R|
    R|2|^^^0127^2202|252.19|U/L|222.0 - 267.0|||F||ML||20130311131745|VistaDim1|
    C|1|I|Q001^45633VB|Q|
    C|2|I|Q004^ASYD MULTIQUAL|Q|
    C|3|I|Q002^201402280000|Q|
    C|4|I|R001^12208BG|R|
    C|5|I|R002^201307262359|R|
    R|3|^^^2101^2202|1.37|g/L|1.34 - 1.61|||F||ML||20130311131658|VistaDim1|
    C|1|I|E550|E|
    C|2|I|Q001^45633VB|Q|
    C|3|I|Q004^ASYD MULTIQUAL|Q|
    C|4|I|Q002^201402280000|Q|
    C|5|I|R001^12292BA|R|
    C|6|I|R002^201310182359|R|
    L|1|N|
    """
    assert message == expected
